export default class GeneratedExpression {
    constructor(expressionTemplate, parameterTypes) {
        this.expressionTemplate = expressionTemplate;
        this.parameterTypes = parameterTypes;
    }
    get source() {
        return format(this.expressionTemplate, ...this.parameterTypes.map((t) => t.name || ''));
    }
    /**
     * Returns an array of parameter names to use in generated function/method signatures
     *
     * @returns {ReadonlyArray.<String>}
     */
    get parameterNames() {
        const usageByTypeName = {};
        return this.parameterTypes.map((t) => getParameterName(t.name || '', usageByTypeName));
    }
}
function getParameterName(typeName, usageByTypeName) {
    let count = usageByTypeName[typeName];
    count = count ? count + 1 : 1;
    usageByTypeName[typeName] = count;
    return count === 1 ? typeName : `${typeName}${count}`;
}
function format(pattern, ...args) {
    return pattern.replace(/{(\d+)}/g, (match, number) => args[number]);
}
//# sourceMappingURL=GeneratedExpression.js.map